namespace MessageAPI.APIData
{
    public class MessageDataRepository : IMessageDataRepository
    {
        private readonly DataContext DataContext;
        private readonly IMapper Mapper;

        public MessageDataRepository(DataContext dataContext, IMapper mapper)
        {
            DataContext = dataContext;
            Mapper = mapper;
        }

        public async Task<Message> GetMessage(int id)
        {
            return await DataContext.Messages.FindAsync(id);
        }

        public async Task<Paging<MessageViewItem>> GetAllMessages(FilterMessages filterMessages)
        {
            var filter = DataContext.Messages.OrderByDescending(p => p.TextSent).AsQueryable(); 
            filter = filterMessages.Container switch
            {
                "Inbox" => filter.Where(p => p.Target.UserName == filterMessages.Username),
                "Outbox" => filter.Where(p => p.Issuer.UserName == filterMessages.Username),
                _ => filter.Where(p => p.Target.UserName == filterMessages.Username && p.TextRead == null) // underscore in switch means default case
            };
            var text = filter.ProjectTo<MessageViewItem>(Mapper.ConfigurationProvider);
            return await Paging<MessageViewItem>.Create(text);
        }

        public async Task<IEnumerable<MessageViewItem>> MessageBetweenUsers(string issuerName, string targetName)
        {
            var text = await DataContext.Messages
                .Where(p => p.Target.UserName == issuerName && p.Issuer.UserName == targetName || p.Target.UserName == targetName && p.Issuer.UserName == issuerName)
                .OrderBy(p => p.TextSent).ToListAsync();
            var uMessages = text.Where( p=> p.TextRead == null && p.TargetUsername == issuerName ).ToList();
            if(uMessages.Any())
            {
                foreach(var texts in uMessages)
                {
                    texts.TextRead = DateTime.UtcNow;
                }
                await DataContext.SaveChangesAsync();
            }
            return Mapper.Map<IEnumerable<MessageViewItem>>(text);
        }

        public void newMessage(Message messages)
        {
            DataContext.Messages.Add(messages);
        }

        public async Task<bool> Save()
        {
            return await DataContext.SaveChangesAsync() > 0;
        }

        public void NewSignalRGroup(SignalRGroup signalRGroup)
        {
            DataContext.SignalRGroups.Add(signalRGroup);
        }

        public void RemoveUsersSignalR(UserSignalR usersSignalR)
        {
            DataContext.UsersSignalRs.Remove(usersSignalR);
        }

        public async Task<UserSignalR> NewUsersSignalR(string id)
        {
            return await DataContext.UsersSignalRs.FindAsync(id);
        }

        public async Task<SignalRGroup> AddNewSignalRGroup(string name)
        {
            return await DataContext.SignalRGroups.Include(t => t.UsersSignalR).FirstOrDefaultAsync( p => p.GroupName == name);
        }

        public void DeleteText(Message messages)
        {
             DataContext.Remove(messages);
        }
    }
}