namespace MessageAPI.APIMiddleware
{
    public class CustomMiddleware
    {
        private readonly RequestDelegate RequestDelegate;
        private readonly ILogger<CustomMiddleware> Logger;
        public CustomMiddleware(RequestDelegate requestDelegate, ILogger<CustomMiddleware> logger)
        {
            Logger = logger;
            RequestDelegate = requestDelegate;
        }

        public async Task InvokeAsync(HttpContext httpContext)
        {
            try
            {
                await RequestDelegate(httpContext);
            }
            catch(Exception exception)
            {
                Logger.LogError(exception, exception.Message);
                httpContext.Response.ContentType = "application/json";
                httpContext.Response.StatusCode = (int) HttpStatusCode.InternalServerError;
                var resp = new ApiErrors(httpContext.Response.StatusCode,"internal server error");
                var serialization = new JsonSerializerOptions{PropertyNamingPolicy = JsonNamingPolicy.CamelCase};
                var result = JsonSerializer.Serialize(resp, serialization);
                await httpContext.Response.WriteAsync(result);
            }
        }
    }
}