namespace MessageAPI.APIServices
{
    public class Token : IToken
    {
        private readonly SymmetricSecurityKey securityKey; // SymmetricSecurityKey is generating one key both for encrypting and decrypting data
        public Token(IConfiguration configuration)
        {
            securityKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(configuration["TokenKey"]));
        }

        public string NewToken(User user)
        {
            var claims = new List<Claim>{
                new Claim(JwtRegisteredClaimNames.NameId, user.UserName)
            };

            var credentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha512Signature);
            var description = new SecurityTokenDescriptor{
                Subject = new ClaimsIdentity(claims),
                Expires = DateTime.Now.AddDays(5),
                SigningCredentials = credentials
            };
            var handler = new JwtSecurityTokenHandler();
            var token = handler.CreateToken(description);
            return handler.WriteToken(token);
        }
    }
}