namespace MessageAPI.Controllers
{
    [ServiceFilter(typeof(wasActive))]
    [Authorize]
    [ApiController]
    [Route("api/[controller]")]
    public class APIUsersController : ControllerBase
    {
        private readonly IUserDataRepository UserDataRepository;
        private readonly IMapper Mapper;
        
        private readonly IPhotoService PhotoService;
        public APIUsersController(IUserDataRepository userDataRepository, IMapper mapper, IPhotoService photoService)
        {
            PhotoService = photoService;
            UserDataRepository = userDataRepository;
            Mapper = mapper;
            
        }

        [HttpGet]
        public async Task<ActionResult<IEnumerable<UserForPhotoReturn>>> GetAllUsers()
        {
            var allUsers = await UserDataRepository.GetAllUsers();
            var returnData = Mapper.Map<IEnumerable<UserForPhotoReturn>>(allUsers);
            return Ok(returnData);
        }


        [HttpGet("{username}", Name = "GetUser")]
        public async Task<ActionResult<UserForPhotoReturn>> GetOneUser(string username)
        {
            var UserByUsername = await UserDataRepository.GetByUsername(username);
            return Mapper.Map<UserForPhotoReturn>(UserByUsername);
        }

        [HttpPut]
        public async Task<ActionResult> Update(UpdateUser user)
        {
            var name = User.FindFirst(ClaimTypes.NameIdentifier)?.Value;
            var mappedUser = await UserDataRepository.GetByUsername(name);
            Mapper.Map(user, mappedUser);
            UserDataRepository.ProfileUpdate(mappedUser);
            if (await UserDataRepository.SaveChanges()) return NoContent();
            return BadRequest("Couldn't update user");
        }

        [HttpPost("newPhoto")]
        public async Task<ActionResult<PhotoReturn>> AddPhoto(IFormFile file)
        {
            var values = await UserDataRepository.GetByUsername(User.FindFirst(ClaimTypes.NameIdentifier)?.Value);
            var response = await PhotoService.PutNewPhoto(file);
            var image = new Photo
            {
                Adress = response.SecureUrl.AbsoluteUri,
                PublicAdress = response.PublicId,
            };

            if (values.Photo.Count == 0)
            {
                image.Leading = true;
            }
            values.Photo.Add(image);
            if (await UserDataRepository.SaveChanges())
            {
                return CreatedAtRoute("GetUser", new { username = values.UserName }, Mapper.Map<PhotoReturn>(image));
            }

            return BadRequest("cannot add this photo!");
        }

        [HttpPut("leadingPhoto/{id}")]
        public async Task<ActionResult> LeadingPhoto(int id)
        {
            var values = await UserDataRepository.GetByUsername(User.FindFirst(ClaimTypes.NameIdentifier)?.Value);
            var image = values.Photo.FirstOrDefault( u => u.Id == id);
            var leading = values.Photo.FirstOrDefault( u => u.Leading);
            if(leading != null) leading.Leading = false;
            image.Leading = true;
            if(await UserDataRepository.SaveChanges()) return NoContent();
            return BadRequest("couldn set leading photo");
        }

        [HttpDelete("photoDelete/{id}")]
        public async Task<ActionResult> PhotoDelete(int id)
        {
            var values = await UserDataRepository.GetByUsername(User.FindFirst(ClaimTypes.NameIdentifier)?.Value);
            var image = values.Photo.FirstOrDefault( u => u.Id == id);
            if(image.Leading) return BadRequest("cannot delete leading photo");

            if(image.PublicAdress != null){
               var delete =  await PhotoService.DeletePhoto(image.PublicAdress);

               if(delete.Error != null) return BadRequest("error"); //make sure to not delete from db if it is not in cloudinary
            }
            values.Photo.Remove(image);
            if(await UserDataRepository.SaveChanges()) return Ok();

            return BadRequest("cannot delete photo");
        }
    }
}