namespace MessageAPI.Controllers
{
    [ServiceFilter(typeof(wasActive))]
    [Authorize]
    [ApiController]
    [Route("api/[controller]")]
    public class MessageController: ControllerBase
    {
        private readonly IUserDataRepository UserDataRepository;
        private readonly IMessageDataRepository MessageDataRepository;
        private readonly IMapper Mapper;
        public MessageController(IUserDataRepository userDataRepository, IMessageDataRepository messageDataRepository, IMapper mapper)
        {
            UserDataRepository = userDataRepository;
            MessageDataRepository = messageDataRepository;
            Mapper = mapper;
        }

        [HttpPost]
        public async Task<ActionResult<MessageViewItem>> newMessage(NewMessageViewItem newMessageViewItem )
        {
            var name = User.FindFirst(ClaimTypes.NameIdentifier)?.Value;
            if(name == newMessageViewItem.TargetUsername) return BadRequest("Cant message to yourself");
            var issuer = await UserDataRepository.GetByUsername(name);
            var target  = await UserDataRepository.GetByUsername(newMessageViewItem.TargetUsername);
            if(target == null) return NotFound();
            var newMessage = new Message
            {
                Issuer = issuer,
                Target = target,
                IssuerUsername = issuer.UserName,
                TargetUsername = target.UserName,
                Text = newMessageViewItem.Text
            };
            MessageDataRepository.newMessage(newMessage);
            if(await MessageDataRepository.Save()) return Ok(Mapper.Map<MessageViewItem>(newMessage));
            return BadRequest("couldn't send message");
        }

        [HttpGet]
        public async Task<ActionResult<IEnumerable<MessageViewItem>>> getAllMessages([FromQuery] FilterMessages filterMessages )
        {
            filterMessages.Username = User.FindFirst(ClaimTypes.NameIdentifier)?.Value;
            var texts = await MessageDataRepository.GetAllMessages(filterMessages);
            return texts;
        }

        [HttpGet("message/{username}")]
        public async Task<ActionResult<IEnumerable<MessageViewItem>>>MessageBetweenUsers(string username)
        {
            var name = User.FindFirst(ClaimTypes.NameIdentifier)?.Value;
            return Ok(await MessageDataRepository.MessageBetweenUsers(name, username.ToUpper()));
        }

        [HttpDelete("{id}")]
        public async Task<IActionResult> DeleteMessage(int id)
        {
            var message = await MessageDataRepository.GetMessage(id);
            if (message == null)
            {
                return NotFound();
            }
            MessageDataRepository.DeleteText(message);
            if (await MessageDataRepository.Save())
            {
                return Ok();
            }

            return BadRequest("Could not delete the message");
        }
    }
}