namespace MessageAPI.Controllers
{
    [ServiceFilter(typeof(wasActive))]
    [ApiController]
    [Route("api/[controller]")]
    public class RegisterController : ControllerBase
    {
        private readonly DataContext UsersDatacontext;
        private readonly IToken Token;
        private readonly IMapper Mapper;
        public RegisterController(DataContext datacontext, IToken token, IMapper mapper)
        {
            Token = token;
            UsersDatacontext = datacontext;
            Mapper = mapper;
        }

        [HttpPost("login")]
        public async Task<ActionResult<ReturnUser>> Login(LoginUser loginUser)
        {
            var user = await UsersDatacontext.Users.SingleOrDefaultAsync(user => user.UserName == loginUser.Username.ToUpper());
            if (user == null) return Unauthorized("The username doesnt exist!");

            using var encryptedPassword = new HMACSHA512(user.Salt);
            var decryptedPassword = encryptedPassword.ComputeHash(Encoding.UTF8.GetBytes(loginUser.Password));
            for (int a = 0; a < decryptedPassword.Length; a++)
            {
                if (decryptedPassword[a] != user.Hash[a]) return Unauthorized("wrong password");
            }

            return new ReturnUser
            {
                Username = user.UserName,
                Token = Token.NewToken(user)
            }; ;
        }

        [HttpPost("register")]
        public async Task<ActionResult<ReturnUser>> RegisterUser(RegisterUser registerUser)
        {

            if (await Existance(registerUser.UserName)) return BadRequest("Username exists, please choose a differnet username");
            var mappedUser = Mapper.Map<User>(registerUser);
            using var encryptedPassword = new HMACSHA512();

                mappedUser.UserName = registerUser.UserName.ToUpper();
                mappedUser.Hash = encryptedPassword.ComputeHash(Encoding.UTF8.GetBytes(registerUser.Password));
                mappedUser.Salt = encryptedPassword.Key;
            
            UsersDatacontext.Users.Add(mappedUser);
            await UsersDatacontext.SaveChangesAsync();

            return new ReturnUser
            {
                Username = mappedUser.UserName,
                Token = Token.NewToken(mappedUser)
            };
        }

        private async Task<bool> Existance(string username)
        {
            return await UsersDatacontext.Users.AnyAsync(user => user.UserName == username.ToUpper());
        }
    }
}