

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
// ordering doesnt mattter in services
builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
                {
                    c.SwaggerDoc("v1", new Microsoft.OpenApi.Models.OpenApiInfo { Title = "Message API V1", Version = "v1" });
                    //other options

                    c.AddSecurityDefinition("Bearer", new Microsoft.OpenApi.Models.OpenApiSecurityScheme()
                    {
                        Description = "JWT Authorization header using the Bearer scheme (Example: 'Bearer token')",
                        Name = "Authorization",
                        In = Microsoft.OpenApi.Models.ParameterLocation.Header,
                        Type = Microsoft.OpenApi.Models.SecuritySchemeType.ApiKey,
                        Scheme = "Bearer"
                    });

                    c.AddSecurityRequirement(new Microsoft.OpenApi.Models.OpenApiSecurityRequirement
                    {
                        {
                            new Microsoft.OpenApi.Models.OpenApiSecurityScheme
                            {
                                Reference = new Microsoft.OpenApi.Models.OpenApiReference
                                {
                                    Type = Microsoft.OpenApi.Models.ReferenceType.SecurityScheme,
                                    Id = "Bearer"
                                }
                            },
                            Array.Empty<string>()
                        }
                    });
                });
//passing expression as a prameter
builder.Services.AddDbContext<DataContext>(options =>
{
    options.
    UseSqlite(builder.Configuration.GetConnectionString("DefaultConnection")); // this is a connection string that connects our application to database that injects data from appsettings.development.json
});

builder.Services.Configure<CloudinarySett>(builder.Configuration.GetSection("CloudinarySettings"));
builder.Services.AddSignalR();
builder.Services.AddScoped<IPhotoService, PhotoService>();
builder
    .Services
    .AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
    .AddJwtBearer(configuringOptions =>
    {
        configuringOptions.TokenValidationParameters = new TokenValidationParameters
        {
            ValidateIssuerSigningKey = true,
            IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(builder.Configuration["TokenKey"])),
            ValidateIssuer = false,
            ValidateAudience = false,
        };

        configuringOptions.Events = new JwtBearerEvents
        {
            OnMessageReceived = config =>
            {
                var signalRtoken = config.Request.Query["access_token"];

                var way = config.HttpContext.Request.Path;

                if (!string.IsNullOrEmpty(signalRtoken) && way.StartsWithSegments("/signalR"))
                {
                    config.Token = signalRtoken;
                }

                return Task.CompletedTask;
            }
        };
    });
builder.Services.AddScoped<IToken, Token>();
builder.Services.AddCors();
builder.Services.AddScoped<IUserDataRepository, UserDataRepository>();
builder.Services.AddAutoMapper(typeof(MappingData).Assembly);
builder.Services.AddScoped<wasActive>();
builder.Services.AddScoped<IMessageDataRepository, MessageDataRepository>();
builder.Services.AddSingleton<OnlineUsersDictionary>();

var app = builder.Build();

app.UseMiddleware<CustomMiddleware>();
app.UseSwagger();
app.UseSwaggerUI(c =>
{
    c.SwaggerEndpoint("/swagger/v1/swagger.json", "MessageApp API V1");
});

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    // app.UseSwagger();
    // app.UseSwaggerUI();
}

app.UseHttpsRedirection();

app.UseRouting();

app.UseCors(policy => policy.AllowAnyHeader().AllowAnyMethod()
        .AllowCredentials().WithOrigins("http://localhost:4200"));

app.UseAuthentication();

app.UseAuthorization();

app.MapControllers();
app.MapHub<OnlineHub>("signalR/online");
app.MapHub<RealTimeMessageHub>("signalR/chat");

using var scope = app.Services.CreateScope();
var Services = scope.ServiceProvider;
try
{
    var context = Services.GetRequiredService<DataContext>();
    await context.Database.MigrateAsync(); //this method will do the migrations automatically when we restart our program
    await SeedData.SeedDatabase(context);

}
catch (Exception ex)
{

    var logger = Services.GetRequiredService<ILogger<Program>>();
    logger.LogError(ex, "Migration did not executed!");
}

await app.RunAsync();


