import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ReplaySubject } from 'rxjs';
import { map } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { ClientUser } from '../clientEntities/clientUser';
import { OnlineService } from './online.service';

//angular servicew is singleton,whn we will start using untill user stops it will e runnoing
@Injectable({
  providedIn: 'root'
})
export class LogSignService {
  private userValues = new ReplaySubject<ClientUser>(1); //persisting login as we store the user in a buffer
  userValues$ = this.userValues.asObservable();
  URL = environment.URLForAPI;

  constructor(private httpRequest: HttpClient, private onlineService: OnlineService) { }

  loginUser(logUser: ClientUser){
    return this.httpRequest.post(this.URL + 'register/login' , logUser).pipe(map((serverResponse: any) => {
      const response = serverResponse;
      if(response){
        localStorage.setItem('user_access', JSON.stringify(response))
        this.userValues.next(response);
        this.onlineService.onlineConnection(response);
      }
    }))
  }

  setUserValues(clientUser: ClientUser){
    localStorage.setItem('user_access', JSON.stringify(clientUser))
    this.userValues.next(clientUser)
  }

  logoutUser(){
    localStorage.removeItem('user_access');
    this.userValues.next(null)
    this.onlineService.stop();
    
  }

  register(clientUser: any){
    return this.httpRequest.post(this.URL+ 'register/register', clientUser)
    .pipe(map((u : ClientUser) =>{ if(u) {localStorage.setItem('user_access', JSON.stringify(u));
   this.userValues.next(u);
   this.onlineService.onlineConnection(u);
  }} ))
  }
}
