import { Injectable } from '@angular/core';
import { HubConnection, HubConnectionBuilder } from '@microsoft/signalr';
import { ToastrService } from 'ngx-toastr';
import { BehaviorSubject } from 'rxjs';
import { environment } from 'src/environments/environment';
import { ClientUser } from '../clientEntities/clientUser';

@Injectable({
  providedIn: 'root'
})
export class OnlineService {
  URL = environment.URLForSignalR;
  private connection: HubConnection;
  private dictionaryUser  = new BehaviorSubject<string[]>([]);
  dictionary$ = this.dictionaryUser.asObservable();
  constructor(private toastr: ToastrService) {}

  onlineConnection(clientUser: ClientUser){
    //creating the signalR connection with authenticated user
    this.connection = new HubConnectionBuilder().withUrl(this.URL + 'online', {
      accessTokenFactory: () => clientUser.token
    }).withAutomaticReconnect().build()
    //start the connection
    this.connection.start()

    this.connection.on('ThisUserIsOnline', name =>{ //the method name string has to match exactly as in backend service for signal r
      this.toastr.success('The user ' + name +  ' is online.' );
    })

    this.connection.on('ThisUserIsOffline', name =>{
      this.toastr.info('The user ' + name + ' is offline');
    })

    this.connection.on('allOnline', (allOnlineNames: string[]) =>{
      this.dictionaryUser.next(allOnlineNames);
    }  )
  }

  stop(){
    this.connection.stop();
  }
}
