import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { map, take } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { clientMessage } from '../clientEntities/clientMessage';
import { HubConnection, HubConnectionBuilder } from '@microsoft/signalr';
import { ClientUser } from '../clientEntities/clientUser';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class TextService {
  URL = environment.URLForAPI;
  SignalRUrl = environment.URLForSignalR;

  private connection: HubConnection;

  private clientText = new BehaviorSubject<clientMessage[]>([]);
  clientText$ = this.clientText.asObservable();

  constructor(private httpRequest : HttpClient) { }

  messagesBetweenUsers(username: string){
    return this.httpRequest.get<clientMessage[]>(this.URL + 'Message/message/' + username)
  }

  deleteText(textId: number){
    return this.httpRequest.delete(this.URL + 'Message/' + textId)
  }

  allMessages(container){
    var parameters = new HttpParams().append('Container', container);
    return this.httpRequest.get<clientMessage[]>(this.URL + 'Message',{observe:'response', params: parameters}).pipe(map(r => {return r}))
  }

  async messagetoUser(username: string, text:string){
    return this.connection.invoke('MessageWithUser',{targetUsername: username, text})
  }

  onlineConnection(clientUser : ClientUser, targetUsername: string){
    this.connection = new HubConnectionBuilder().withUrl(this.SignalRUrl + 'chat?APIUser=' + targetUsername,{
      accessTokenFactory: () => clientUser.token
    }).withAutomaticReconnect().build()

    this.connection.start().catch(error => console.log(error));

    this.connection.on('GetMessage', chat =>{
      this.clientText.next(chat);
    } )

    this.connection.on('NewChat', chat =>{
      this.clientText$.pipe(take(1)).subscribe(texts => {
        this.clientText.next([...texts, chat]);
      })
    } )
  }

  stop(){
    if(this.connection){
      this.connection.stop();
    }
    
  }

}
