import { Component, Input, OnDestroy, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { TabDirective, TabsetComponent } from 'ngx-bootstrap/tabs';
import { clientMessage } from 'src/app/clientEntities/clientMessage';
import { MappedUsersService } from 'src/app/clientServices/mapped-users.service';
import { TextService } from 'src/app/clientServices/text.service';
import { MappedUsers } from 'src/app/clientViewItems/mappedUsers';
import { Photo } from 'src/app/clientViewItems/photo';
import { OnlineService } from 'src/app/clientServices/online.service';
import { LogSignService } from 'src/app/clientServices/log-sign.service';
import { ClientUser } from 'src/app/clientEntities/clientUser';
import { take } from 'rxjs';
import { ElementRef } from '@angular/core';

@Component({
  selector: 'app-detail',
  templateUrl: './detail.component.html',
  styleUrls: ['./detail.component.css'],
  encapsulation: ViewEncapsulation.None
})
export class DetailComponent implements OnInit, OnDestroy {
  @ViewChild('detailTab', { static: true }) detailTab: TabsetComponent;
  @ViewChild('scrollMe', { static: false }) private scrollContainer: ElementRef;
  tabDirective: TabDirective; //comes with ngx-bootstrap
  text: clientMessage[] = [];
  user: MappedUsers;
  gallery: Photo[];
  paramText: string;
  clientUser: ClientUser

  constructor(public onlineService: OnlineService, private mappedUsersService: MappedUsersService,
    private activatedRoute: ActivatedRoute, public textService: TextService, private logservice: LogSignService) {
    this.logservice.userValues$.pipe(take(1)).subscribe(clientUser => this.clientUser = clientUser);
  }


  ngOnDestroy(): void {
    this.textService.stop();
  }

  ngOnInit(): void {
    this.getUser();
    this.activatedRoute.queryParams.subscribe(p => p.tab ? this.activateMessageTab(p.tab) : this.activateMessageTab(0));
  }

  messageToOneUser() {
    this.textService.messagetoUser(this.user?.userName, this.paramText);
    this.scrollContainer.nativeElement.scrollTop = this.scrollContainer.nativeElement.scrollHeight;
    this.paramText ='';
  }

  activateMessageTab(id: number) {
    this.detailTab.tabs[id].active = true;
  }

  tab(directive: TabDirective) {
    this.tabDirective = directive;

  }

  messagesBetweenUsers() {
    this.textService.messagesBetweenUsers(this.user?.userName).subscribe(texts => this.text = texts)
  }

  userPhotos() {
    for (const photo of this.user.photo) {
      this.gallery = [{
        id: photo.id,
        leading: photo.leading,
        adress: photo.adress
      }]
    }
  }

  getUser() {
    this.mappedUsersService.getMappedUser(this.activatedRoute.snapshot.paramMap.get('username')).subscribe(e => {
      this.user = e;
      if (this.user) {
        this.userPhotos();
        this.messagesBetweenUsers();
        this.textService.onlineConnection(this.clientUser, this.user.userName);
      }
    })
  }

}
