import { Component, Input, OnInit } from '@angular/core';
import { FileUploader } from 'ng2-file-upload';
import { take } from 'rxjs';
import { ClientUser } from 'src/app/clientEntities/clientUser';
import { LogSignService } from 'src/app/clientServices/log-sign.service';
import { MappedUsersService } from 'src/app/clientServices/mapped-users.service';
import { MappedUsers } from 'src/app/clientViewItems/mappedUsers';
import { Photo } from 'src/app/clientViewItems/photo';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-photo-config',
  templateUrl: './photo-config.component.html',
  styleUrls: ['./photo-config.component.css']
})
export class PhotoConfig implements OnInit {
 @Input() mappedUser: MappedUsers
 uploadConfig: FileUploader;
 
  URL = environment.URLForAPI;
  clientUser: ClientUser;

  constructor(private MappedUserService: MappedUsersService ,private loginService: LogSignService) { 
    this.loginService.userValues$.pipe(take(1)).subscribe(user => this.clientUser = user);
  }

  ngOnInit(): void {
    this.Upload();
  }

  photoDelete(id){
    this.MappedUserService.photoDelete(id).subscribe(() =>{
      this.mappedUser.photo = this.mappedUser.photo.filter(i => i.id !== id)
    })

  }
  leading(image: Photo){
    console.log(image.id)
    this.MappedUserService.leadingPhoto(image.id).subscribe(()=>{
      //this.clientUser.photoAdress = image.adress;
      //this.loginService.setUserValues(this.clientUser);
      this.mappedUser.photoAdress = image.adress;
      console.log(image)
      this.mappedUser.photo.forEach(i =>{
        if(i.leading) i.leading = false;
        if(i.id === image.id) i.leading = true;
      })
    })
  }


  Upload(){
    this.uploadConfig = new FileUploader({
      url: this.URL + 'APIUsers/newPhoto',
      authToken: 'Bearer ' + this.clientUser.token,
      isHTML5: true,
      allowedFileType: ['image'],
      removeAfterUpload: true,
      autoUpload: false,
      maxFileSize: 10 * 1024 * 1024
    })

    this.uploadConfig.onAfterAddingFile =  (file) =>{
      file.withCredentials = false;
    }

    this.uploadConfig.onSuccessItem = (item, response, status, headers) => {
      if(response){
        const photo = JSON.parse(response)
        this.mappedUser.photo.push(photo)
      }
    }
  }


}
